From mboxrd@z Thu Jan  1 00:00:00 1970
Date: Wed, 23 Feb 2022 01:16:34 +0000
To: "list+cgit@causal.agency" <list+cgit@causal.agency>
From: equa <equaa@protonmail.com>
Reply-To: equa <equaa@protonmail.com>
Subject: [PATCH] Add "default-tab" and "root-default-tab" configuration options
Message-ID: <kWwJNIGstU-kTBgV2TSyWeWnT8SD-OuYNo56FmNZ5wlKBct4-u_GbcOV6KYc5uv2iRYuMzoSyIRhs2DnVjTgpTW8XPJbAiKLaxMOwUoZlh8=@protonmail.com>
In-Reply-To: <za0y1QaWB3dEbfM4JzSkr3RvHv8tr7lStOoAFlokRkGI7AHpwWEVsyOQxzrg6eINpcKaSCcgqK-4w3bIhb9ty2piIqpEysqEtJg-jKvtQks=@protonmail.com>
References: <za0y1QaWB3dEbfM4JzSkr3RvHv8tr7lStOoAFlokRkGI7AHpwWEVsyOQxzrg6eINpcKaSCcgqK-4w3bIhb9ty2piIqpEysqEtJg-jKvtQks=@protonmail.com>
MIME-Version: 1.0
Content-Type: text/plain; charset=utf-8
Content-Transfer-Encoding: quoted-printable

These options allow the user to specify a page to display at the root
repository/index location instead of the default summary or repository list=
.

Signed-off-by: equa <equaa@protonmail.com>
---
 cgit.c        | 10 ++++++++++
 cgit.h        |  3 +++
 cgitrc.5.txt  | 11 +++++++++++
 cmd.c         |  6 +++---
 shared.c      |  1 +
 ui-repolist.c |  2 +-
 ui-shared.c   | 24 ++++++++++++++++++++----
 ui-shared.h   |  2 ++
 8 files changed, 51 insertions(+), 8 deletions(-)

diff --git a/cgit.c b/cgit.c
index 08d81a1..f1c2224 100644
--- a/cgit.c
+++ b/cgit.c
@@ -56,6 +56,8 @@ static void repo_config(struct cgit_repo *repo, const cha=
r *name, const char *va
 =09=09repo->homepage =3D xstrdup(value);
 =09else if (!strcmp(name, "defbranch"))
 =09=09repo->defbranch =3D xstrdup(value);
+=09else if (!strcmp(name, "default-tab"))
+=09=09repo->default_tab =3D xstrdup(value);
 =09else if (!strcmp(name, "extra-head-content"))
 =09=09repo->extra_head_content =3D xstrdup(value);
 =09else if (!strcmp(name, "snapshots"))
@@ -285,6 +287,10 @@ static void config_cb(const char *name, const char *va=
lue)
 =09=09ctx.cfg.clone_url =3D xstrdup(value);
 =09else if (!strcmp(name, "local-time"))
 =09=09ctx.cfg.local_time =3D atoi(value);
+=09else if (!strcmp(name, "default-tab"))
+=09=09ctx.cfg.default_tab =3D xstrdup(value);
+=09else if (!strcmp(name, "root-default-tab"))
+=09=09ctx.cfg.root_default_tab =3D xstrdup(value);
 =09else if (!strcmp(name, "commit-sort")) {
 =09=09if (!strcmp(value, "date"))
 =09=09=09ctx.cfg.commit_sort =3D 1;
@@ -373,6 +379,8 @@ static void prepare_context(void)
 =09ctx.cfg.cache_scanrc_ttl =3D 15;
 =09ctx.cfg.cache_dynamic_ttl =3D 5;
 =09ctx.cfg.cache_static_ttl =3D -1;
+=09ctx.cfg.default_tab =3D "summary";
+=09ctx.cfg.root_default_tab =3D "repolist";
 =09ctx.cfg.case_sensitive_sort =3D 1;
 =09ctx.cfg.branch_sort =3D 0;
 =09ctx.cfg.commit_sort =3D 0;
@@ -816,6 +824,8 @@ static void print_repo(FILE *f, struct cgit_repo *repo)
 =09=09fprintf(f, "repo.homepage=3D%s\n", repo->homepage);
 =09if (repo->clone_url)
 =09=09fprintf(f, "repo.clone-url=3D%s\n", repo->clone_url);
+=09if (repo->default_tab)
+=09=09fprintf(f, "repo.default-tab=3D%s\n", repo->default_tab);
 =09fprintf(f, "repo.enable-blame=3D%d\n",
 =09        repo->enable_blame);
 =09fprintf(f, "repo.enable-commit-graph=3D%d\n",
diff --git a/cgit.h b/cgit.h
index 69b5c13..bb555cf 100644
--- a/cgit.h
+++ b/cgit.h
@@ -86,6 +86,7 @@ struct cgit_repo {
 =09char *owner;
 =09char *homepage;
 =09char *defbranch;
+=09char *default_tab;
 =09char *module_link;
 =09struct string_list readme;
 =09char *section;
@@ -215,6 +216,8 @@ struct cgit_config {
 =09char *repository_sort;
 =09char *virtual_root;=09/* Always ends with '/'. */
 =09char *strict_export;
+=09char *default_tab;
+=09char *root_default_tab;
 =09int cache_size;
 =09int cache_dynamic_ttl;
 =09int cache_max_create_time;
diff --git a/cgitrc.5.txt b/cgitrc.5.txt
index 33a6a8c..f32fb33 100644
--- a/cgitrc.5.txt
+++ b/cgitrc.5.txt
@@ -128,6 +128,10 @@ css::
 =09Url which specifies the css document to include in all cgit pages.
 =09Default value: "/cgit.css".

+default-tab::
+=09Specifies the tab to automatically load in the root page of a repositor=
y.
+=09Default value: "summary". See also: root-default-tab, repo.default-tab.
+
 email-filter::
 =09Specifies a command which will be invoked to format names and email
 =09address of committers, authors, and taggers, as represented in various
@@ -352,6 +356,10 @@ robots::
 =09Text used as content for the "robots" meta-tag. Default value:
 =09"index, nofollow".

+root-default-tab::
+=09Specifies the tab to automatically load as the root index page.
+=09Default value: "repolist". See also: default-tab, repo.default-tab.
+
 root-desc::
 =09Text printed below the heading on the repository index page. Default
 =09value: "a fast webinterface for the git dscm".
@@ -481,6 +489,9 @@ repo.defbranch::
 =09as default instead. Default value: branch pointed to by HEAD, or
 =09"master" if there is no suitable HEAD.

+repo.default-tab::
+=09Override the global default-tab. Default value: none.
+
 repo.desc::
 =09The value to show as repository description. Default value: none.

diff --git a/cmd.c b/cmd.c
index 0eb75b1..64f6639 100644
--- a/cmd.c
+++ b/cmd.c
@@ -55,7 +55,7 @@ static void about_fn(void)
 =09=09=09cgit_redirect(ctx.repo->homepage, false);
 =09=09else {
 =09=09=09char *currenturl =3D cgit_currenturl();
-=09=09=09char *redirect =3D fmtalloc("%s../", currenturl);
+=09=09=09char *redirect =3D cgit_pageurl(ctx.repo->url, "summary", NULL);
 =09=09=09cgit_redirect(redirect, false);
 =09=09=09free(currenturl);
 =09=09=09free(redirect);
@@ -196,9 +196,9 @@ struct cgit_cmd *cgit_get_cmd(void)

 =09if (ctx.qry.page =3D=3D NULL) {
 =09=09if (ctx.repo)
-=09=09=09ctx.qry.page =3D "summary";
+=09=09=09ctx.qry.page =3D ctx.repo->default_tab;
 =09=09else
-=09=09=09ctx.qry.page =3D "repolist";
+=09=09=09ctx.qry.page =3D ctx.cfg.root_default_tab;
 =09}

 =09for (i =3D 0; i < sizeof(cmds)/sizeof(*cmds); i++)
diff --git a/shared.c b/shared.c
index 8115469..b1090a6 100644
--- a/shared.c
+++ b/shared.c
@@ -79,6 +79,7 @@ struct cgit_repo *cgit_add_repo(const char *url)
 =09ret->clone_url =3D ctx.cfg.clone_url;
 =09ret->submodules.strdup_strings =3D 1;
 =09ret->hide =3D ret->ignore =3D 0;
+=09ret->default_tab =3D ctx.cfg.default_tab;
 =09return ret;
 }

diff --git a/ui-repolist.c b/ui-repolist.c
index 529a203..75db72a 100644
--- a/ui-repolist.c
+++ b/ui-repolist.c
@@ -321,7 +321,7 @@ void cgit_print_repolist(void)
 =09=09}
 =09=09htmlf("<tr><td class=3D'%s'>",
 =09=09      !sorted && section ? "sublevel-repo" : "toplevel-repo");
-=09=09cgit_summary_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
+=09=09cgit_repo_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
 =09=09html("</td><td>");
 =09=09repourl =3D cgit_repourl(ctx.repo->url);
 =09=09html_link_open(repourl, NULL, NULL);
diff --git a/ui-shared.c b/ui-shared.c
index acd8ab5..17b1e49 100644
--- a/ui-shared.c
+++ b/ui-shared.c
@@ -203,6 +203,9 @@ static void site_url(const char *page, const char *sear=
ch, const char *sort, int
 {
 =09char *delim =3D "?";

+=09if (!strcmp(page ? page : "", ctx.cfg.root_default_tab))
+=09=09page =3D NULL;
+
 =09if (always_root || page)
 =09=09html_attr(cgit_rooturl());
 =09else {
@@ -257,7 +260,7 @@ static void site_link(const char *page, const char *nam=
e, const char *title,
 void cgit_index_link(const char *name, const char *title, const char *clas=
s,
 =09=09     const char *pattern, const char *sort, int ofs, int always_root=
)
 {
-=09site_link(NULL, name, title, class, pattern, sort, ofs, always_root);
+=09site_link("repolist", name, title, class, pattern, sort, ofs, always_ro=
ot);
 }

 static char *repolink(const char *title, const char *class, const char *pa=
ge,
@@ -317,6 +320,12 @@ static void reporevlink(const char *page, const char *=
name, const char *title,
 {
 =09char *delim;

+=09if (page
+=09    && !rev
+=09    && !path
+=09    && !strcmp(page, ctx.repo->default_tab))
+=09=09page =3D NULL;
+
 =09delim =3D repolink(title, class, page, head, path);
 =09if (rev && ctx.qry.head !=3D NULL && strcmp(rev, ctx.qry.head)) {
 =09=09html(delim);
@@ -328,10 +337,17 @@ static void reporevlink(const char *page, const char =
*name, const char *title,
 =09html("</a>");
 }

+void cgit_repo_link(const char *name, const char *title, const char *class=
,
+=09=09    const char *head)
+{
+=09reporevlink(NULL, name, title, class, head, NULL, NULL);
+}
+
+
 void cgit_summary_link(const char *name, const char *title, const char *cl=
ass,
 =09=09       const char *head)
 {
-=09reporevlink(NULL, name, title, class, head, NULL, NULL);
+=09reporevlink("summary", name, title, class, head, NULL, NULL);
 }

 void cgit_tag_link(const char *name, const char *title, const char *class,
@@ -995,7 +1011,7 @@ static void print_header(void)
 =09if (ctx.repo) {
 =09=09cgit_index_link("index", NULL, NULL, NULL, NULL, 0, 1);
 =09=09html(" : ");
-=09=09cgit_summary_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
+=09=09cgit_repo_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
 =09=09if (ctx.env.authenticated) {
 =09=09=09html("</td><td class=3D'form'>");
 =09=09=09html("<form method=3D'get'>\n");
@@ -1084,7 +1100,7 @@ void cgit_print_pageheader(void)
 =09=09html("</form>\n");
 =09} else if (ctx.env.authenticated) {
 =09=09char *currenturl =3D cgit_currenturl();
-=09=09site_link(NULL, "index", NULL, hc("repolist"), NULL, NULL, 0, 1);
+=09=09site_link("repolist", "index", NULL, hc("repolist"), NULL, NULL, 0, =
1);
 =09=09if (ctx.cfg.root_readme)
 =09=09=09site_link("about", "about", NULL, hc("about"),
 =09=09=09=09  NULL, NULL, 0, 1);
diff --git a/ui-shared.h b/ui-shared.h
index 6964873..9faf2ba 100644
--- a/ui-shared.h
+++ b/ui-shared.h
@@ -17,6 +17,8 @@ extern void cgit_add_clone_urls(void (*fn)(const char *))=
;

 extern void cgit_index_link(const char *name, const char *title,
 =09=09=09    const char *class, const char *pattern, const char *sort, int=
 ofs, int always_root);
+extern void cgit_repo_link(const char *name, const char *title,
+=09=09=09   const char *class, const char *head);
 extern void cgit_summary_link(const char *name, const char *title,
 =09=09=09      const char *class, const char *head);
 extern void cgit_tag_link(const char *name, const char *title,
--
2.30.2

From mboxrd@z Thu Jan  1 00:00:00 1970
Date: Wed, 23 Feb 2022 01:40:47 +0000
To: "equaa@protonmail.com" <equaa@protonmail.com>
From: equa <equaa@protonmail.com>
Cc: "list+cgit@causal.agency" <list+cgit@causal.agency>
Reply-To: equa <equaa@protonmail.com>
Subject: Re: [PATCH] Add "default-tab" and "root-default-tab" configuration options
Message-ID: <cic_aaDSjdiq_DFx-wYcxEuYG8iPAQlw8BWrJWzwpkcd68aCT9mjaJmEP6NmL5ZSxpB5ZWoGsbQEZXVmz4EjzTj2xsfvAZfW1Tqr1KzCnG0=@protonmail.com>
MIME-Version: 1.0
Content-Type: text/plain; charset=utf-8
Content-Transfer-Encoding: quoted-printable

There are a few problems with this patch in retrospect, I think.
I intended to make the default page show up at the empty url but
gracefully fall back to the full URL (for example, a link to the
tree, if the tree was the default page, would be linked to as
just "repo/", but the files within it would be "repo/tree/files".)
This passes initial inspection but it doesn't work for about pages
that have subpages: since the output HTML isn't filtered at all
relative links will based the repository root. It may be better
to turn this into a redirect instead, which would also mean pages
are more consistent even if the defaults change.

From mboxrd@z Thu Jan  1 00:00:00 1970
Content-Type: text/plain;
	charset=us-ascii
Mime-Version: 1.0 (Mac OS X Mail 13.4 \(3608.120.23.2.7\))
Subject: Re: [PATCH] Add "default-tab" and "root-default-tab" configuration
 options
From: june <june@causal.agency>
In-Reply-To: <cic_aaDSjdiq_DFx-wYcxEuYG8iPAQlw8BWrJWzwpkcd68aCT9mjaJmEP6NmL5ZSxpB5ZWoGsbQEZXVmz4EjzTj2xsfvAZfW1Tqr1KzCnG0=@protonmail.com>
Date: Tue, 1 Mar 2022 19:22:08 -0500
Cc: "list+cgit@causal.agency" <list+cgit@causal.agency>
Content-Transfer-Encoding: 7bit
Message-Id: <6B2AC072-DEE4-4C32-A694-B8492A2DCA73@causal.agency>
References: <cic_aaDSjdiq_DFx-wYcxEuYG8iPAQlw8BWrJWzwpkcd68aCT9mjaJmEP6NmL5ZSxpB5ZWoGsbQEZXVmz4EjzTj2xsfvAZfW1Tqr1KzCnG0=@protonmail.com>
To: equa <equaa@protonmail.com>

> On Feb 22, 2022, at 20:40, equa <equaa@protonmail.com> wrote:
> 
> There are a few problems with this patch in retrospect, I think.
> I intended to make the default page show up at the empty url but
> gracefully fall back to the full URL (for example, a link to the
> tree, if the tree was the default page, would be linked to as
> just "repo/", but the files within it would be "repo/tree/files".)
> This passes initial inspection but it doesn't work for about pages
> that have subpages: since the output HTML isn't filtered at all
> relative links will based the repository root. It may be better
> to turn this into a redirect instead, which would also mean pages
> are more consistent even if the defaults change.

Hm, it's a shame about the about pages. I guess redirects are the
better solution, though it's a little disappointing.


From mboxrd@z Thu Jan  1 00:00:00 1970
Date: Fri, 29 Apr 2022 01:33:49 +0000
To: june <june@causal.agency>
From: equa <equaa@protonmail.com>
Cc: "list+cgit@causal.agency" <list+cgit@causal.agency>
Reply-To: equa <equaa@protonmail.com>
Subject: Re: [PATCH] Add "default-tab" and "root-default-tab" configuration options
Message-ID: <BVScjTFTtTRbxLBuyET776DWoC18Fg_F_-ZnO9ytF06L3Yf6XLa9oYB4tsowv0P2ivLfMO1lk--plhQK5E_qIIxjB1FU3THyojuecfzwFW8=@protonmail.com>
In-Reply-To: <6B2AC072-DEE4-4C32-A694-B8492A2DCA73@causal.agency>
References: <cic_aaDSjdiq_DFx-wYcxEuYG8iPAQlw8BWrJWzwpkcd68aCT9mjaJmEP6NmL5ZSxpB5ZWoGsbQEZXVmz4EjzTj2xsfvAZfW1Tqr1KzCnG0=@protonmail.com> <6B2AC072-DEE4-4C32-A694-B8492A2DCA73@causal.agency>
MIME-Version: 1.0
Content-Type: text/plain; charset=utf-8
Content-Transfer-Encoding: quoted-printable

(messed up cc on last email, apologies.)

a possibility is modifying the filters instead. we could
pass as an argument the base URL for any links the filters
encounter. i'd have to see if this is feasible to add to the
existing filters, but i'm curious what you'd think of such an
implementation.

From mboxrd@z Thu Jan  1 00:00:00 1970
Content-Type: text/plain;
	charset=us-ascii
Mime-Version: 1.0 (Mac OS X Mail 16.0 \(3696.80.82.1.1\))
Subject: Re: [PATCH] Add "default-tab" and "root-default-tab" configuration
 options
From: june <june@causal.agency>
In-Reply-To: <BVScjTFTtTRbxLBuyET776DWoC18Fg_F_-ZnO9ytF06L3Yf6XLa9oYB4tsowv0P2ivLfMO1lk--plhQK5E_qIIxjB1FU3THyojuecfzwFW8=@protonmail.com>
Date: Sat, 14 May 2022 14:11:48 -0400
Cc: "list+cgit@causal.agency" <list+cgit@causal.agency>
Content-Transfer-Encoding: 7bit
Message-Id: <F1B6E36D-DEC4-44D2-A039-2D182E51E423@causal.agency>
References: <cic_aaDSjdiq_DFx-wYcxEuYG8iPAQlw8BWrJWzwpkcd68aCT9mjaJmEP6NmL5ZSxpB5ZWoGsbQEZXVmz4EjzTj2xsfvAZfW1Tqr1KzCnG0=@protonmail.com>
 <6B2AC072-DEE4-4C32-A694-B8492A2DCA73@causal.agency>
 <BVScjTFTtTRbxLBuyET776DWoC18Fg_F_-ZnO9ytF06L3Yf6XLa9oYB4tsowv0P2ivLfMO1lk--plhQK5E_qIIxjB1FU3THyojuecfzwFW8=@protonmail.com>
To: equa <equaa@protonmail.com>

> On Apr 28, 2022, at 21:33, equa <equaa@protonmail.com> wrote:
> 
> (messed up cc on last email, apologies.)
> 
> a possibility is modifying the filters instead. we could
> pass as an argument the base URL for any links the filters
> encounter. i'd have to see if this is feasible to add to the
> existing filters, but i'm curious what you'd think of such an
> implementation.

That's actually a great idea! If that works it would also make it
possible to render the README on the summary page itself which would
be really neat.


